"""
This file was copied from
https://github.com/TaylorSMarks/playsound
playsound.py - For playing audio file, Copyright (c) 2016 Taylor Marks
MIT License
----
I've added async play for linux using a thread, changed names to be more pythonic
"""

from platform import system

system = system()


class PlayException(Exception):
    pass


def _play_sound_win(sound, block=True):
    """
    Utilizes windll.winmm. Tested and known to work with MP3 and WAVE on
    Windows 7 with Python 2.7. Probably works with more file formats.
    Probably works on Windows XP thru Windows 10. Probably works with all
    versions of Python.
    Inspired by (but not copied from) Michael Gundlach <gundlach@gmail.com>'s mp3play:
    https://github.com/michaelgundlach/mp3play
    I never would have tried using windll.winmm without seeing his code.
    """
    from ctypes import c_buffer, windll
    from random import random
    from time import sleep
    from sys import getfilesystemencoding

    def win_cmd(*command):
        buf = c_buffer(255)
        command = " ".join(command).encode(getfilesystemencoding())
        err_code = int(windll.winmm.mciSendStringA(command, buf, 254, 0))
        if err_code:
            error_buffer = c_buffer(255)
            windll.winmm.mciGetErrorStringA(err_code, error_buffer, 254)
            exception_message = (
                "\n    Error " + str(err_code) + " for command:"
                "\n        " + command.decode() + "\n    " + error_buffer.value.decode()
            )
            raise PlayException(exception_message)
        return buf.value

    alias = "playsound_" + str(random())
    win_cmd('open "' + sound + '" alias', alias)
    win_cmd("set", alias, "time format milliseconds")
    duration_ms = win_cmd("status", alias, "length")
    win_cmd("play", alias, "from 0 to", duration_ms.decode())

    if block:
        sleep(float(duration_ms) / 1000.0)


def _play_sound_osx(sound, block=True):
    """
    Utilizes AppKit.NSSound. Tested and known to work with MP3 and WAVE on
    OS X 10.11 with Python 2.7. Probably works with anything QuickTime supports.
    Probably works on OS X 10.5 and newer. Probably works with all versions of
    Python.
    Inspired by (but not copied from) Aaron's Stack Overflow answer here:
    http://stackoverflow.com/a/34568298/901641
    I never would have tried using AppKit.NSSound without seeing his code.
    """
    from AppKit import NSSound
    from Foundation import NSURL
    from time import sleep

    if "://" not in sound:
        if not sound.startswith("/"):
            from os import getcwd

            sound = getcwd() + "/" + sound
        sound = "file://" + sound
    url = NSURL.URLWithString_(sound)
    nssound = NSSound.alloc().initWithContentsOfURL_byReference_(url, True)
    if not nssound:
        raise IOError("Unable to load sound named: " + sound)
    nssound.play()

    if block:
        sleep(nssound.duration())


def _play_sound_nix_blocking(sound):
    """Play a sound using GStreamer.
    Inspired by this:
    https://gstreamer.freedesktop.org/documentation/tutorials/playback/playbin-usage.html
    """
    # pathname2url escapes non-URL-safe characters
    import os

    try:
        from urllib.request import pathname2url
    except ImportError:
        # python 2
        from urllib import pathname2url

    import gi

    gi.require_version("Gst", "1.0")
    from gi.repository import Gst

    Gst.init(None)

    playbin = Gst.ElementFactory.make("playbin", "playbin")
    if sound.startswith(("http://", "https://")):
        playbin.props.uri = sound
    else:
        playbin.props.uri = "file://" + pathname2url(os.path.abspath(sound))

    set_result = playbin.set_state(Gst.State.PLAYING)
    if set_result != Gst.StateChangeReturn.ASYNC:
        raise PlayException("playbin.set_state returned " + repr(set_result))

    bus = playbin.get_bus()
    bus.poll(Gst.MessageType.EOS, Gst.CLOCK_TIME_NONE)
    playbin.set_state(Gst.State.NULL)


def _play_sound_nix_no_except(sound):
    # noinspection PyBroadException
    try:
        _play_sound_nix(sound)
    except:
        pass


def _play_sound_nix(sound, block=True):
    if block:
        _play_sound_nix_blocking(sound)
        return

    from threading import Thread

    thread = Thread(target=_play_sound_nix_no_except, args=(sound,), daemon=True)
    thread.start()


if system == "Windows":
    play = _play_sound_win
elif system == "Darwin":
    play = _play_sound_osx
else:
    play = _play_sound_nix

del system



#!/usr/bin/env python
import threading

from prompt_toolkit.application import Application
from prompt_toolkit.application.current import get_app
from prompt_toolkit.key_binding import KeyBindings
from prompt_toolkit.key_binding.bindings.focus import focus_next, focus_previous
from prompt_toolkit.layout import HSplit, Layout, VSplit, FormattedTextControl, Window
from prompt_toolkit.styles import Style
from prompt_toolkit.widgets import Box, Button, Label

from pydoro.pydoro_core.config import Configuration
from pydoro.pydoro_core.tomato import Tomato
from pydoro.pydoro_core.util import every


class UserInterface:
    def __init__(self, config: Configuration):
        self.config = config
        self.tomato = Tomato(self.config)
        self.prev_hash = None

        self._create_ui()

    def _create_ui(self):
        btn_start = Button("Start", handler=self.tomato.start)
        btn_pause = Button("Pause", handler=self.tomato.pause)
        btn_reset = Button("Reset", handler=self.tomato.reset)
        btn_reset_all = Button("Reset All", handler=self.tomato.reset_all)
        btn_exit = Button("Exit", handler=self._exit_clicked)
        # All the widgets for the UI.
        self.text_area = FormattedTextControl(focusable=False, show_cursor=False)
        text_window = Window(
            content=self.text_area,
            dont_extend_height=True,
            height=11,
            style="bg:#ffffff #000000",
        )
        root_container = Box(
            HSplit(
                [
                    Label(text="Press `Tab` to move the focus."),
                    HSplit(
                        [
                            VSplit(
                                [
                                    btn_start,
                                    btn_pause,
                                    btn_reset,
                                    btn_reset_all,
                                    btn_exit,
                                ],
                                padding=1,
                                style="bg:#cccccc",
                            ),
                            text_window,
                        ]
                    ),
                ]
            )
        )
        layout = Layout(container=root_container, focused_element=btn_start)
        self._set_key_bindings()

        # Styling.
        style = Style(
            [
                ("left-pane", "bg:#888800 #000000"),
                ("right-pane", "bg:#00aa00 #000000"),
                ("button", "#000000"),
                ("button-arrow", "#000000"),
                ("button focused", "bg:#ff0000"),
                ("red", "#ff0000"),
                ("green", "#00ff00"),
            ]
        )
        self.application = Application(
            layout=layout, key_bindings=self.kb, style=style, full_screen=True
        )

    def _set_key_bindings(self):
        self.kb = KeyBindings()

        actions = {
            "focus_next": focus_next,
            "focus_previous": focus_previous,
            "exit_clicked": self._exit_clicked,
            "start": lambda _=None: self.tomato.start(),
            "pause": lambda _=None: self.tomato.pause(),
            "reset": lambda _=None: self.tomato.reset(),
            "reset_all": lambda _=None: self.tomato.reset_all(),
        }

        for action, keys in self.config.key_bindings.items():
            for key in keys.split(","):
                try:
                    self.kb.add(key.strip())(actions[action])
                except KeyError:
                    pass

    @staticmethod
    def _exit_clicked(_=None):
        get_app().exit()

    def _draw(self):
        self.tomato.update()
        text, hash_ = self.tomato.render()
        # WHY: Avoid unnecessary updates
        if hash_ != self.prev_hash:
            self.text_area.text = text
            self.application.invalidate()
            self.prev_hash = hash_

    def run(self):
        self._draw()
        threading.Thread(target=lambda: every(0.4, self._draw), daemon=True).start()
        self.application.run()


def main():
    UserInterface(Configuration()).run()


if __name__ == "__main__":
    main()



import itertools
import sys
from enum import IntEnum
from timeit import default_timer

from pydoro.pydoro_core import sound
from pydoro.pydoro_core.util import in_app_path
from pydoro.pydoro_core.config import Configuration

SECONDS_PER_MIN = 60

PLACEHOLDER_TIME = "time"
PLACEHOLDER_STATUS = "status"
PLACEHOLDER_SETS = "sets"
PLACEHOLDER_COUNT = "count"
PLACEHOLDER_TASK = "task1", "task2", "task3", "task4"

NO_COLOUR = ""
RED = "#ff0000"
GREEN = "#00cc00"
BOLD_TEXT = "bold"

# Tomato ascii art
#   tuples is made of colour and a piece of coloured text
TOMATO = [
    (NO_COLOUR, "\n      "),
    (GREEN, "/'\\/`\\"),
    (NO_COLOUR, "         "),
    (NO_COLOUR, PLACEHOLDER_TASK[0]),
    (NO_COLOUR, "\n    "),
    (RED, ".-"),
    (NO_COLOUR, "  "),
    (GREEN, "|/"),
    (NO_COLOUR, "  "),
    (RED, "-."),
    (NO_COLOUR, "       "),
    (NO_COLOUR, PLACEHOLDER_TASK[1]),
    (NO_COLOUR, "\n   "),
    (RED, "/"),
    (NO_COLOUR, "          "),
    (RED, "\\"),
    (NO_COLOUR, "      "),
    (NO_COLOUR, PLACEHOLDER_TASK[2]),
    (NO_COLOUR, "\n  "),
    (RED, "'"),
    (NO_COLOUR, "   "),
    (BOLD_TEXT, "pydoro"),
    (NO_COLOUR, "   "),
    (RED, "\\"),
    (NO_COLOUR, "     "),
    (NO_COLOUR, PLACEHOLDER_TASK[3]),
    (NO_COLOUR, "\n "),
    (RED, ";"),
    (NO_COLOUR, "             "),
    (RED, "'"),
    (NO_COLOUR, "     "),
    (NO_COLOUR, PLACEHOLDER_STATUS),
    (NO_COLOUR, "\n "),
    (RED, ";"),
    (NO_COLOUR, "             "),
    (RED, ";"),
    (NO_COLOUR, "     "),
    (NO_COLOUR, PLACEHOLDER_TIME),
    (NO_COLOUR, "\n "),
    (RED, ":"),
    (NO_COLOUR, "          "),
    (RED, "/"),
    (NO_COLOUR, "  "),
    (RED, "."),
    (NO_COLOUR, "\n  "),
    (RED, "\\"),
    (NO_COLOUR, "       "),
    (RED, ".'"),
    (NO_COLOUR, "  "),
    (RED, "/"),
    (NO_COLOUR, "      "),
    (NO_COLOUR, PLACEHOLDER_COUNT),
    (NO_COLOUR, "\n    "),
    (RED, "\\"),
    (NO_COLOUR, " "),
    (RED, "____"),
    (NO_COLOUR, " "),
    (RED, ".'"),
    (NO_COLOUR, "        "),
    (NO_COLOUR, PLACEHOLDER_SETS),
]

LOCATIONS = {
    PLACEHOLDER_COUNT: 51,
    PLACEHOLDER_SETS: 59,
    PLACEHOLDER_STATUS: 31,
    PLACEHOLDER_TASK[0]: 3,
    PLACEHOLDER_TASK[1]: 11,
    PLACEHOLDER_TASK[2]: 17,
    PLACEHOLDER_TASK[3]: 25,
    PLACEHOLDER_TIME: 37,
}

TEXT_LONG_BREAK = r"""
  ___    LONG      _
 | _ )_ _ ___ __ _| |__
 | _ | '_/ -_/ _` | / /
 |___|_| \___\__,_|_\_\
""".strip(
    "\r\n"
)

TEXT_SMALL_BREAK = TEXT_LONG_BREAK.replace("LONG", "SMALL")

TEXT_WORK = r"""
 __      __       _
 \ \    / ___ _ _| |__
  \ \/\/ / _ | '_| / /
   \_/\_/\___|_| |_\_\
""".strip(
    "\r\n"
)


class Tasks(IntEnum):
    WORK = 1
    SMALL_BREAK = 2
    LONG_BREAK = 3
    NO_TASK = 4
    INTERMEDIATE = 5


class TaskStatus(IntEnum):
    NONE = 111
    STARTED = 222
    PAUSED = 555
    LIMBO = 666


TEXT = {
    TaskStatus.NONE: "",
    TaskStatus.STARTED: "",
    TaskStatus.PAUSED: "PAUSED",
    TaskStatus.LIMBO: "",
    Tasks.WORK: TEXT_WORK,
    Tasks.SMALL_BREAK: TEXT_SMALL_BREAK,
    Tasks.LONG_BREAK: TEXT_LONG_BREAK,
    Tasks.NO_TASK: "",
    Tasks.INTERMEDIATE: "",
}

PROGRESS = ["|#  |", "| # |", "|  #|", "| # |"]


def cur_time():
    return int(default_timer())


class InitialState:
    name = "initial"

    def __init__(self, tomato):
        self._tomato = tomato
        self._task = Tasks.NO_TASK
        self._status = TaskStatus.NONE
        self._started_at = 0
        self._remainder = 0
        self._time_period = 0
        self._progress = itertools.cycle(PROGRESS)

    def start(self):
        self._tomato.play_alarm()
        return WorkingState(self._tomato)

    def pause(self):
        return self

    def reset(self):
        return self

    @property
    def remainder(self):
        return self._remainder

    @property
    def next_state(self):
        return self

    @property
    def time_period(self):
        return self._time_period

    @property
    def time_remaining(self):
        return "Press [start]"

    @property
    def task(self):
        return self._task

    @property
    def status(self):
        return self._status

    @property
    def done(self):
        return False

    def _format_time(self, remainder):
        minutes, seconds = divmod(int(remainder), SECONDS_PER_MIN)
        if self.status == TaskStatus.STARTED:
            progress = next(self._progress) + " "
        else:
            progress = ""

        return "{}{:00}min {:00}s remaining".format(progress, minutes, seconds)

    def _calc_remainder(self):
        cur = cur_time()
        self._remainder = max(self._remainder - (cur - self._started_at), 0)
        self._started_at = cur


class IntermediateState(InitialState):
    name = "waiting"

    def __init__(self, tomato):
        super().__init__(tomato)
        self._task = Tasks.INTERMEDIATE
        self._status = TaskStatus.LIMBO
        self._next_factory = None
        self._last_alarm_time = 0
        self._sound()

    def _sound(self):
        if cur_time() - self._last_alarm_time > self._tomato.configs.alarm_seconds:
            self._tomato.play_alarm()
            self._last_alarm_time = cur_time()

    def start(self):
        return self._next_factory(self._tomato)

    @property
    def time_remaining(self):
        self._sound()
        return "Press [start] to continue with " + self._next_factory.name

    @property
    def done(self):
        return False

    @staticmethod
    def transition_to(next_state_factory, tomato):
        state = IntermediateState(tomato)
        state._next_factory = next_state_factory
        return state


class WorkingState(InitialState):
    name = "work"

    def __init__(self, tomato):
        super().__init__(tomato)
        self._time_period = self._remainder = int(
            self._tomato.configs.work_minutes * SECONDS_PER_MIN
        )
        self._task = Tasks.WORK
        self._status = TaskStatus.STARTED
        self._started_at = cur_time()

    def start(self):
        return self

    @property
    def time_remaining(self):
        self._calc_remainder()
        if self._tomato.no_clock:
            return ""
        return self._format_time(self._remainder)

    @property
    def next_state(self):
        self._tomato.tomatoes += 1
        if self._tomato.tomatoes % self._tomato.tomatoes_per_set == 0:
            return IntermediateState.transition_to(LongBreakState, self._tomato)
        return IntermediateState.transition_to(SmallBreakState, self._tomato)

    def pause(self):
        return WorkPausedState.return_to(self._tomato, self)

    def reset(self):
        self._remainder = self.time_period
        return self

    @property
    def done(self):
        return self._remainder <= 0


class WorkPausedState(InitialState):
    name = "work paused"

    def __init__(self, tomato):
        super().__init__(tomato)
        self._prev = None
        self._task = Tasks.WORK
        self._status = TaskStatus.PAUSED

    def start(self):
        self._prev._started_at = cur_time()
        return self._prev

    def reset(self):
        self._prev._remainder = self._prev.time_period
        return self

    @property
    def time_remaining(self):
        return self._format_time(self._prev.remainder)

    @staticmethod
    def return_to(tomato, state):
        cur_state = WorkPausedState(tomato)
        cur_state._prev = state
        return cur_state

    @property
    def done(self):
        return False


class SmallBreakState(InitialState):
    name = "small break"

    def __init__(self, tomato):
        super().__init__(tomato)
        self._time_period = self._remainder = int(
            self._tomato.configs.small_break_minutes * SECONDS_PER_MIN
        )
        self._task = Tasks.SMALL_BREAK
        self._status = TaskStatus.STARTED
        self._started_at = cur_time()

    def start(self):
        return self

    @property
    def time_remaining(self):
        self._calc_remainder()

        return self._format_time(self._remainder)

    @property
    def next_state(self):
        return IntermediateState.transition_to(WorkingState, self._tomato)

    def pause(self):
        return SmallBreakPausedState.return_to(self._tomato, self)

    def reset(self):
        self._remainder = self.time_period
        return self

    @property
    def done(self):
        return self._remainder <= 0


class SmallBreakPausedState(InitialState):
    name = "small break paused"

    def __init__(self, tomato):
        super().__init__(tomato)
        self._task = Tasks.SMALL_BREAK
        self._status = TaskStatus.PAUSED
        self._prev = None

    def start(self):
        self._prev._started_at = cur_time()
        return self._prev

    @property
    def time_remaining(self):
        return self._format_time(self._prev.remainder)

    @staticmethod
    def return_to(tomato, state):
        cur_state = SmallBreakPausedState(tomato)
        cur_state._prev = state
        return cur_state

    def reset(self):
        self._prev._remainder = self._prev.time_period
        return self

    @property
    def done(self):
        return False


class LongBreakState(SmallBreakState):
    name = "long break"

    def __init__(self, tomato):
        super().__init__(tomato)
        self._time_period = self._remainder = int(
            self._tomato.configs.long_break_minutes * SECONDS_PER_MIN
        )
        self._task = Tasks.LONG_BREAK
        self._status = TaskStatus.STARTED

    def pause(self):
        return LongBreakPausedState.return_to(self._tomato, self)


class LongBreakPausedState(SmallBreakPausedState):
    name = "long break paused"

    def __init__(self, tomato):
        super().__init__(tomato)
        self._task = Tasks.LONG_BREAK
        self._status = TaskStatus.PAUSED

    @staticmethod
    def return_to(tomato, state):
        cur_state = LongBreakPausedState(tomato)
        cur_state._prev = state
        return cur_state


class Tomato:
    def __init__(self, configs: Configuration):
        self.configs = configs
        self.no_sound = configs.no_sound
        self.emoji = configs.emoji
        self.tomatoes_per_set = configs.tomatoes_per_set
        self.no_clock = configs.no_clock
        self.tomatoes = 0

        self._state = InitialState(self)

    def start(self):
        self._state = self._state.start()

    def pause(self):
        self._state = self._state.pause()

    def reset(self):
        self._state = self._state.reset()

    def reset_all(self):
        self._state = InitialState(self)
        self.tomatoes = 0

    def update(self):
        if self._state.done:
            self._state = self._state.next_state

    def play_alarm(self):
        if self.no_sound:
            return
        # noinspection PyBroadException
        try:
            sound.play(in_app_path("b15.wav"), block=False)
        except Exception:
            pass

    def tomato_symbol(self):
        ascii_tomato = "(`) "
        if self.emoji:
            try:
                "🍅".encode(sys.stdout.encoding)
                return "🍅 "
            except UnicodeEncodeError:
                return ascii_tomato
        return ascii_tomato

    def render(self) -> (list, int):
        task = TEXT[self._state.task]
        task = task.splitlines()
        if not task:
            task = [""] * 4

        sets = self.tomatoes // self.tomatoes_per_set
        if sets == 1:
            sets = "1 set completed"
        elif sets >= 2:
            sets = str(sets) + " sets completed"
        else:
            sets = ""

        status = TEXT[self._state.status]
        time = self._state.time_remaining
        count = self.tomato_symbol() * (
            self.tomatoes_per_set - self.tomatoes % self.tomatoes_per_set
        )

        ftext = TOMATO[:]
        for i in range(0, 4):
            ftext[LOCATIONS[PLACEHOLDER_TASK[i]]] = ("", task[i])
        ftext[LOCATIONS[PLACEHOLDER_STATUS]] = ("", status)
        ftext[LOCATIONS[PLACEHOLDER_TIME]] = ("", time)
        ftext[LOCATIONS[PLACEHOLDER_COUNT]] = ("", count)
        ftext[LOCATIONS[PLACEHOLDER_SETS]] = ("", sets)

        hash_ = hash((status, time, count, sets) + tuple(task))

        return ftext, hash_



from codecs import open
from inspect import getsource
from os.path import abspath, dirname, join

from setuptools import setup, find_packages

here = abspath(dirname(getsource(lambda: 0)))

with open(join(here, "README.rst"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="pydoro",
    version="0.2.1",
    # Get the description from second line & remove `*` character
    description=long_description.splitlines()[2][1:-1],
    long_description=long_description,
    long_description_content_type="text/x-rst",
    url="https://github.com/JaDogg/pydoro",
    author="Bhathiya Perera",
    author_email="jadogg.coder@gmail.com",
    python_requires=">=3.6",
    license="MIT",
    package_data={"": ["*.rst", "*.wav"]},
    install_requires=["prompt-toolkit>=3.0.3"],
    extras_require={
        'audio:platform_system=="Darwin"': [
            "pyobjc-core>=5.2",
            "pyobjc-framework-Cocoa>=5.2",
        ],
        'audio:platform_system=="Linux"': ["pycairo>=1.18.1", "PyGObject>=3.32.1"],
    },
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Information Technology",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    keywords="tomato pomodoro pydoro timer work",
    packages=find_packages(),
    entry_points={"console_scripts": ["pydoro = pydoro.pydoro_tui:main"]},
    setup_requires=["wheel"],
)



import argparse
import configparser
import os


class Configuration:
    def __init__(self):
        self._cli_parse()
        self._ini_parse()
        self._ini_load()
        self._cli_load()

    def _cli_parse(self):
        """
        Parse command line arguments
        """
        parser = argparse.ArgumentParser(
            "pydoro", description="Terminal Pomodoro Timer"
        )
        parser.add_argument(
            "-e",
            "--emoji",
            action="store_true",
            help="If set, use tomato emoji instead of the ASCII art",
        )
        parser.add_argument(
            "--focus",
            help="focus mode: hides clock and \
                            mutes sounds (equivalent to --no-clock and --no-sound)",
            action="store_true",
        )
        parser.add_argument("--no-clock", help="hides clock", action="store_true")
        parser.add_argument("--no-sound", help="mutes all sounds", action="store_true")
        self.cli_args = parser.parse_args()

    def _ini_parse(self):
        """
        Parse configuration file
        Look at PYDORO_CONFIG_FILE environment variable
        Defaults to ~/.pydoro.ini if PYDORO_CONFIG_FILE not set
        """
        self._conf = configparser.ConfigParser()

        filename = os.environ.get(
            "PYDORO_CONFIG_FILE", os.path.expanduser("~/.pydoro.ini")
        )

        if os.path.exists(filename):
            self._conf.read(filename)
        else:
            self._create_default_ini()

    def _create_default_ini(self):
        """
        Creates default ini configuration file
        Saves it in '~/.pydoro.ini'
        """
        self._conf["DEFAULT"] = {}

        self._conf["General"] = {}
        self._conf["General"]["no_clock"] = "False"
        self._conf["General"]["no_sound"] = "False"
        self._conf["General"]["emoji"] = "False"

        self._conf["Time"] = {}
        self._conf["Time"]["tomatoes_per_set"] = "4"
        self._conf["Time"]["work_minutes"] = "25"
        self._conf["Time"]["small_break_minutes"] = "5"
        self._conf["Time"]["long_break_minutes"] = "15"
        self._conf["Time"]["alarm_seconds"] = "20"

        self._conf["KeyBindings"] = {}
        self._conf["KeyBindings"]["focus_previous"] = "s-tab,left,h,j"
        self._conf["KeyBindings"]["focus_next"] = "tab,right,l,k"
        self._conf["KeyBindings"]["exit_clicked"] = "q"
        self._conf["KeyBindings"]["start"] = "s"
        self._conf["KeyBindings"]["pause"] = "p"
        self._conf["KeyBindings"]["reset"] = "r"
        self._conf["KeyBindings"]["reset_all"] = "a"

        filename = os.path.expanduser("~/.pydoro.ini")
        with open(filename, "w+") as configfile:
            self._conf.write(configfile)

    def _ini_load(self):
        """
        Loads the .ini config file preferences

        Command line arguments override file configurations.
        """
        self.no_clock = self._conf["General"]["no_clock"] == "True"
        self.no_sound = self._conf["General"]["no_sound"] == "True"
        self.emoji = self._conf["General"]["emoji"] == "True"
        self.tomatoes_per_set = int(self._conf["Time"]["tomatoes_per_set"])
        self.work_minutes = float(self._conf["Time"]["work_minutes"])
        self.small_break_minutes = float(self._conf["Time"]["small_break_minutes"])
        self.long_break_minutes = float(self._conf["Time"]["long_break_minutes"])
        self.alarm_seconds = int(self._conf["Time"]["alarm_seconds"])
        self.key_bindings = self._conf["KeyBindings"]

    def _cli_load(self):
        """
        Loads the command line arguments

        Command line arguments override file configurations.
        """
        self.no_clock = self.cli_args.no_clock or self.cli_args.focus or self.no_clock
        self.no_sound = self.cli_args.no_sound or self.cli_args.focus or self.no_sound
        self.emoji = self.cli_args.emoji or self.emoji
